@@ -22,8 +22,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gphoto2-library.h>
-#include <gphoto2-result.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-result.h>
 #include <bayer.h>
 #include <gamma.h>
 
@@ -52,7 +52,7 @@ camera_id (CameraText *id)
 }
 
 
-struct smal_cameras {
+static const struct smal_cameras {
 	char * name;
 	unsigned short idVendor;
 	unsigned short idProduct;
