@@ -1,22 +1,23 @@
-/** \file gphoto2-camera.c
- * \brief Implement Camera object representing a camera attached to the system.
+/** \file
  *
- * \author Copyright  2000 Scott Fritzinger
- * \author 2001-2002: Lutz Mller <lutz@users.sf.net>
+ * Implement Camera object representing a camera attached to the system.
  *
- * \par
+ * \author Copyright 2000 Scott Fritzinger
+ * \author Copyright 2001-2002 Lutz Mller <lutz@users.sf.net>
+ *
+ * \note
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * \par
+ * \note
  * This library is distributed in the hope that it will be useful, 
  * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details. 
  *
- * \par
+ * \note
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
@@ -24,7 +25,7 @@
  */
 
 #include "config.h"
-#include "gphoto2-camera.h"
+#include <gphoto2/gphoto2-camera.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -33,10 +34,9 @@
 
 #include <ltdl.h>
 
-#include "gphoto2-context.h"
-#include "gphoto2-result.h"
-#include "gphoto2-library.h"
-#include "gphoto2-port-log.h"
+#include <gphoto2/gphoto2-result.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -72,21 +72,21 @@
 
 #define CR(c,result,ctx)						\
 {									\
-	int r = (result);						\
+	int r1 = (result);						\
 									\
-	if (r < 0) {							\
+	if (r1 < 0) {							\
 									\
 		/* libgphoto2_port doesn't have a GPContext */		\
-		if (r > -100)						\
+		if (r1 > -100)						\
 			gp_context_error ((ctx), _("An error occurred "	\
 				"in the io-library ('%s'): %s"),	\
-				gp_port_result_as_string (r),		\
+				gp_port_result_as_string (r1),		\
 				(c) ? gp_port_get_error ((c)->port) :	\
 				      _("No additional information "	\
 				      "available."));			\
 		if (c)							\
 			CAMERA_UNUSED((c),(ctx));			\
-		return (r);						\
+		return (r1);						\
 	}								\
 }
 
@@ -121,100 +121,94 @@
 #ifdef HAVE_MULTI
 #define CHECK_OPEN(c,ctx)						\
 {									\
-	int r;								\
+	int r2;								\
 									\
 	if (strcmp ((c)->pc->a.model,"Directory Browse")) {		\
-		r = gp_port_open ((c)->port);				\
-		if (r < 0) {						\
+		r2 = gp_port_open ((c)->port);				\
+		if (r2 < 0) {						\
 			CAMERA_UNUSED (c,ctx);				\
-			return (r);					\
+			return (r2);					\
 		}							\
 	}								\
 	if ((c)->functions->pre_func) {					\
-		r = (c)->functions->pre_func (c,ctx);			\
-		if (r < 0) {						\
+		r2 = (c)->functions->pre_func (c,ctx);			\
+		if (r2 < 0) {						\
 			CAMERA_UNUSED (c,ctx);				\
-			return (r);					\
+			return (r2);					\
 		}							\
 	}								\
 }
 #else
 #define CHECK_OPEN(c,ctx)						\
 {                                                                       \
-	int r;								\
-									\
-	if ((c)->functions->pre_func) {                                 \
-		r = (c)->functions->pre_func (c,ctx);                   \
-		if (r < 0) {                                            \
+	if ((c)->functions->pre_func) {					\
+		int r2 = (c)->functions->pre_func (c,ctx);		\
+		if (r2 < 0) {                                           \
 			CAMERA_UNUSED (c,ctx);				\
-			return (r);                                     \
+			return (r2);                                    \
 		}                                                       \
 	}                                                               \
 }
 #endif
 
 #ifdef HAVE_MULTI
-#define CHECK_CLOSE(c,ctx)						\
-{									\
-	int r;								\
-									\
-	if (strcmp ((c)->pc->a.model,"Directory Browse"))		\
-		gp_port_close ((c)->port);				\
-	if ((c)->functions->post_func) {				\
-		r = (c)->functions->post_func (c,ctx);			\
-		if (r < 0) {						\
-			CAMERA_UNUSED (c,ctx);				\
-			return (r);					\
-		}							\
-	}								\
+#define CHECK_CLOSE(c,ctx)					\
+{								\
+	if (strcmp ((c)->pc->a.model,"Directory Browse"))	\
+		gp_port_close ((c)->port);			\
+	if ((c)->functions->post_func) {			\
+		int r3 = (c)->functions->post_func (c,ctx);	\
+		if (r3 < 0) {					\
+			CAMERA_UNUSED (c,ctx);			\
+			return (r3);				\
+		}						\
+	}							\
 }
 #else
-#define CHECK_CLOSE(c,ctx)                                              \
-{                                                                       \
-	int r;								\
-									\
-	if ((c)->functions->post_func) {                                \
-		r = (c)->functions->post_func (c,ctx);                  \
-		if (r < 0) {                                            \
-			CAMERA_UNUSED (c,ctx);				\
-			return (r);                                     \
-		}							\
-	}                                                               \
+#define CHECK_CLOSE(c,ctx)					\
+{								\
+	if ((c)->functions->post_func) {			\
+		int r3 = (c)->functions->post_func (c,ctx);	\
+		if (r3 < 0) {					\
+			CAMERA_UNUSED (c,ctx);			\
+			return (r3);				\
+		}						\
+	}							\
 }
 #endif
 
 #define CRS(c,res,ctx)							\
 {									\
-	int r = (res);							\
+	int r4 = (res);							\
 									\
-	if (r < 0) {							\
+	if (r4 < 0) {							\
 		CAMERA_UNUSED (c,ctx);                              	\
-		return (r);						\
+		return (r4);						\
 	}								\
 }
 
 #define CRSL(c,res,ctx,list)						\
 {									\
-	int r = (res);							\
+	int r5 = (res);							\
 									\
-	if (r < 0) {							\
+	if (r5 < 0) {							\
 		CAMERA_UNUSED (c,ctx);                              	\
 		gp_list_free (list);					\
-		return (r);						\
+		return (r5);						\
 	}								\
 }
 
 #define CHECK_RESULT_OPEN_CLOSE(c,result,ctx)				\
 {									\
-	int r;								\
+	int r6;								\
 									\
 	CHECK_OPEN (c,ctx);						\
-	r = (result);							\
-	if (r < 0) {							\
+	r6 = (result);							\
+	if (r6 < 0) {							\
 		CHECK_CLOSE (c,ctx);					\
 		gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Operation failed!");\
 		CAMERA_UNUSED (c,ctx);                              	\
-		return (r);						\
+		return (r6);						\
 	}								\
 	CHECK_CLOSE (c,ctx);						\
 }
@@ -257,10 +251,11 @@ struct _CameraPrivateCore {
 
 
 /**
- * \brief Close connection to camera.
- * \param camera a Camera object
- * \param context a GPContext object
- * \return a gphoto2 error code.
+ * Close connection to camera.
+ *
+ * @param camera a #Camera object
+ * @param context a #GPContext object
+ * @return a gphoto2 error code.
  *
  * Closes a connection to the camera and therefore gives other application
  * the possibility to access the camera, too.
@@ -269,6 +264,7 @@ struct _CameraPrivateCore {
  * call this function when you currently don't need the camera. The camera
  * will get reinitialized by gp_camera_init() automatically if you try to 
  * access the camera again.
+ *
  */
 int
 gp_camera_exit (Camera *camera, GPContext *context)
@@ -316,9 +312,11 @@ gp_camera_exit (Camera *camera, GPContext *context)
 
 
 /**
- * \brief Allocates the memory for a Camera.
- * \param camera The Camera:: object to initialize.
- * \return a gphoto2 error code
+ * Allocates the memory for a #Camera.
+ *
+ * @param camera the #Camera object to initialize.
+ * @return a gphoto2 error code
+ *
  */
 int
 gp_camera_new (Camera **camera)
@@ -368,16 +366,18 @@ gp_camera_new (Camera **camera)
 
 /**
  * \brief Sets the camera abilities. 
- * \param camera: a #Camera
- * \param abilities the #CameraAbilities to be set
- * \return a gphoto2 error code
  *
- * You need to call this function before calling gp_camera_init() the
+ * @param camera a #Camera
+ * @param abilities the #CameraAbilities to be set
+ * @return a gphoto2 error code
+ *
+ * You need to call this function before calling #gp_camera_init the
  * first time unless you want gphoto2 to autodetect cameras and choose
  * the first detected one. By setting the \c abilities, you 
  * tell gphoto2 what model the \c camera is and what camera driver should 
  * be used for accessing the \c camera. You can get \c abilities by calling
- * gp_abilities_list_get_abilities().
+ * #gp_abilities_list_get_abilities.
+ *
  */
 int
 gp_camera_set_abilities (Camera *camera, CameraAbilities abilities)
@@ -401,10 +401,12 @@ gp_camera_set_abilities (Camera *camera, CameraAbilities abilities)
 
 
 /**
- * \brief Retrieve the \c abilities of the \c camera. 
- * \param camera a #Camera
- * \param abilities
- * \return a gphoto2 error code
+ * Retrieve the \c abilities of the \c camera. 
+ *
+ * @param camera a #Camera
+ * @param abilities
+ * @return a gphoto2 error code
+ *
  */
 int
 gp_camera_get_abilities (Camera *camera, CameraAbilities *abilities)
@@ -449,10 +451,11 @@ gp_camera_set_port_info (Camera *camera, GPPortInfo info)
 
 
 /**
- * \brief Set the camera speed.
- * \param camera a Camera
- * \param speed the speed
- * \return a gphoto2 error code
+ * Set the camera speed.
+ *
+ * @param camera a #Camera
+ * @param speed the speed
+ * @return a gphoto2 error code
  *
  * This function is typically used prior first initialization 
  * using #gp_camera_init for debugging purposes. Normally, a camera driver
@@ -461,6 +464,7 @@ gp_camera_set_port_info (Camera *camera, GPPortInfo info)
  * serial ports. In other words, you have to set the camera's port to a 
  * serial one (using #gp_camera_set_port_path or #gp_camera_set_port_name)
  * prior calling this function.
+ *
  */
 int
 gp_camera_set_port_speed (Camera *camera, int speed)
@@ -498,9 +502,11 @@ gp_camera_set_port_speed (Camera *camera, int speed)
 
 
 /** 
- * \brief Retrieve the current speed.
- * \param camera a Camera
- * \return The current speed or a gphoto2 error code
+ * Retrieve the current speed.
+ *
+ * @param camera a #Camera
+ * @return The current speed or a gphoto2 error code
+ *
  */
 int
 gp_camera_get_port_speed (Camera *camera)
@@ -512,9 +518,11 @@ gp_camera_get_port_speed (Camera *camera)
 
 
 /**
- * \brief Increment the reference count of a @camera.
- * \param camera a Camera
- * \return a gphoto2 error code
+ * Increment the reference count of a \c camera.
+ *
+ * @param camera a #Camera
+ * @return a gphoto2 error code
+ *
  */
 int
 gp_camera_ref (Camera *camera)
@@ -528,12 +536,14 @@ gp_camera_ref (Camera *camera)
 
 
 /**
- * \brief Decrements the reference count of a Camera.
- * \param camera a #Camera
- * \return a gphoto2 error code
+ * Decrements the reference count of a #Camera.
+ *
+ * @param camera a #Camera
+ * @return a gphoto2 error code
  *
  * If the reference count reaches %0, the \c camera will be freed 
  * automatically.
+ *
  */
 int
 gp_camera_unref (Camera *camera)
@@ -561,12 +571,14 @@ gp_camera_unref (Camera *camera)
 
 
 /**
- * \brief Free the \c camera.
- * \param camera a Camera
- * \return a gphoto2 error code
+ * Free the \c camera.
+ *
+ * @param camera a #Camera
+ * @return a gphoto2 error code
  *
  * \deprecated 
- * This function should never be used. Please use gp_camera_unref() instead.
+ * This function should never be used. Please use #gp_camera_unref instead.
+ *
  */
 int
 gp_camera_free (Camera *camera)
@@ -612,18 +624,20 @@ gp_camera_free (Camera *camera)
 
 
 /**
- * \brief Initiate a connection to the \c camera. 
- * \param camera a camera
- * \param context a GPContext
- * \return a gphoto2 error code
+ * Initiate a connection to the \c camera. 
+ *
+ * @param camera a #Camera
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
  * Before calling this function, the
- * @camera should be set up using #gp_camera_set_port_path or
+ * \c camera should be set up using #gp_camera_set_port_path or
  * #gp_camera_set_port_name and #gp_camera_set_abilities. If that has been
  * omitted, gphoto2 tries to autodetect any cameras and chooses the first one
  * if any cameras are found. It is generally a good idea to call
  * #gp_camera_exit after transactions have been completed in order to give
  * other applications the chance to access the camera, too.
+ *
  */
 int
 gp_camera_init (Camera *camera, GPContext *context)
@@ -694,7 +708,7 @@ gp_camera_init (Camera *camera, GPContext *context)
 		switch (camera->port->type) {
 		case GP_PORT_NONE:
 			gp_context_error (context, _("You have to set the "
-				"port prior initialization of the camera."));
+				"port prior to initialization of the camera."));
 			return (GP_ERROR_UNKNOWN_PORT);
 		case GP_PORT_USB:
 			if (gp_port_usb_find_device (camera->port,
@@ -767,13 +781,15 @@ gp_camera_init (Camera *camera, GPContext *context)
 
 
 /**
- * \brief Retrieve a configuration \c window for the \c camera.
- * \param camera a Camera
- * \param window a CameraWidget
- * \param context a GPContext
- * \return gphoto2 error code
+ * Retrieve a configuration \c window for the \c camera.
+ *
+ * @param camera a #Camera
+ * @param window a #CameraWidget
+ * @param context a #GPContext
+ * @return gphoto2 error code
  *
  * This \c window can be used for construction of a configuration dialog.
+ *
  */
 int
 gp_camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
@@ -797,16 +813,16 @@ gp_camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 
 
 /**
- * gp_camera_set_config:
- * @camera: a #Camera
- * @window: a #CameraWidget
- * @context: a #GPContext
+ * Sets the configuration.
  *
- * Sets the configuration. Typically, a @window is retrieved using
- * #gp_camera_get_config and passed to this function in order to adjust
- * the settings on the camera.
+ * @param camera a #Camera
+ * @param window a #CameraWidget
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
+ * Typically, a \c window is retrieved using #gp_camera_get_config and passed 
+ * to this function in order to adjust the settings on the camera.
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
@@ -829,16 +845,16 @@ gp_camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 }
 
 /**
- * gp_camera_get_summary:
- * @camera: a #Camera
- * @summary: a #CameraText
- * @context: a #GPContext
+ * Retrieves a camera summary.
  *
- * Retrieves a camera @summary. This summary typically contains information
- * like manufacturer, pictures taken, or generally information that is
- * not configurable.
+ * @param camera a #Camera
+ * @param summary a #CameraText
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
+ * This summary typically contains information like manufacturer, pictures
+ * taken, or generally information that is not configurable.
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_get_summary (Camera *camera, CameraText *summary, GPContext *context)
@@ -861,15 +877,15 @@ gp_camera_get_summary (Camera *camera, CameraText *summary, GPContext *context)
 }
 
 /**
- * gp_camera_get_manual:
- * @camera: a #Camera
- * @manual: a #CameraText
- * @context: a #GPContext
+ * Retrieves the \c manual for given \c camera.
  *
- * Retrieves the @manual for given @camera. This manual typically contains
- * information about using the camera.
+ * @param camera a #Camera
+ * @param manual a #CameraText
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
+ * This manual typically contains information about using the camera.
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_get_manual (Camera *camera, CameraText *manual, GPContext *context)
@@ -892,15 +908,16 @@ gp_camera_get_manual (Camera *camera, CameraText *manual, GPContext *context)
 }
 
 /**
- * gp_camera_get_about:
- * @camera: a #Camera
- * @about: a #CameraText
- * @context: a #GPContext
+ * Retrieves information about the camera driver.
  *
- * Retrieves information about the camera driver. Typically, this information
- * contains name and address of the author, acknowledgements, etc.
+ * @param camera a #Camera
+ * @param about a #CameraText
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
+ * Typically, this information contains name and address of the author,
+ * acknowledgements, etc.
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_get_about (Camera *camera, CameraText *about, GPContext *context)
@@ -923,17 +940,17 @@ gp_camera_get_about (Camera *camera, CameraText *about, GPContext *context)
 }
 
 /**
- * gp_camera_capture:
- * @camera: a #Camera
- * @type: a #CameraCaptureType
- * @path: a #CameraFilePath
- * @context: a #GPContext
+ * Captures an image, movie, or sound clip depending on the given \c type.
+ *
+ * @param camera a #Camera
+ * @param type a #CameraCaptureType
+ * @param path a #CameraFilePath
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
- * Captures an image, movie, or sound clip depending on the given @type.
  * The resulting file will be stored on the camera. The location gets stored
- * in @path. The file can then be downloaded using #gp_camera_file_get.
+ * in \c path. The file can then be downloaded using #gp_camera_file_get.
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_capture (Camera *camera, CameraCaptureType type,
@@ -956,16 +973,17 @@ gp_camera_capture (Camera *camera, CameraCaptureType type,
 }
 
 /**
- * gp_camera_capture_preview:
- * \param camera: a #Camera
- * \param file: a #CameraFile
- * \param context: a #GPContext
- *
  * Captures a preview that won't be stored on the camera but returned in 
- * supplied @file. For example, you could use #gp_capture_preview for 
- * taking some sample pictures before calling #gp_capture.
+ * supplied file. 
+ *
+ * @param camera a #Camera
+ * @param file a #CameraFile
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
+ * For example, you could use #gp_capture_preview for taking some sample
+ * pictures before calling #gp_capture.
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
@@ -991,13 +1009,14 @@ gp_camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 
 
 /**
- * \brief Wait for an event from the \c camera.
- * \param camera a Camera
- * \param timeout amount of time to wait in 1/1000 seconds
- * \param eventtype received CameraEventType [out]
- * \param eventdata received event specific data [out]
- * \param context a GPContext
- * \return gphoto2 error code
+ * Wait for an event from the camera.
+ *
+ * @param camera a Camera
+ * @param timeout amount of time to wait in 1/1000 seconds
+ * @param eventtype received CameraEventType [out]
+ * @param eventdata received event specific data [out]
+ * @param context a GPContext
+ * @return gphoto2 error code
  *
  * This function blocks and waits for an event to come from the camera.  If
  * timeout occurs before an event is received then
@@ -1005,6 +1024,7 @@ gp_camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
  * If an event is received then eventtype is set to the type of event, and
  * eventdata is set to event specific data.  See the CameraEventType enum
  * to see which eventtype's match to which types of eventdata.
+ *
  */
 int
 gp_camera_wait_for_event (Camera *camera, int timeout,
@@ -1028,15 +1048,14 @@ gp_camera_wait_for_event (Camera *camera, int timeout,
 }
 
 /**
- * gp_camera_folder_list_files:
- * \param camera: a #Camera
- * \param folder: a folder
- * \param list: a #CameraList
- * \param context: a #GPContext
+ * Lists the files in supplied \c folder.
  *
- * Lists the files in supplied @folder.
+ * @param camera a #Camera
+ * @param folder a folder
+ * @param list a #CameraList
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_folder_list_files (Camera *camera, const char *folder, 
@@ -1058,15 +1077,14 @@ gp_camera_folder_list_files (Camera *camera, const char *folder,
 }
 
 /**
- * gp_camera_folder_list_folders:
- * \param camera: a #Camera
- * \param folder: a folder
- * \param list: a #CameraList
- * \param context: a #GPContext
+ * Lists the folders in supplied \c folder.
  *
- * Lists the folders in supplied @folder.
+ * @param camera a #Camera
+ * @param folder a folder
+ * @param list a #CameraList
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_folder_list_folders (Camera *camera, const char* folder, 
@@ -1088,14 +1106,13 @@ gp_camera_folder_list_folders (Camera *camera, const char* folder,
 }
 
 /**
- * gp_camera_folder_delete_all:
- * \param camera: a #Camera
- * \param folder: a folder
- * \param context: a #GPContext
+ * Deletes all files in a given \c folder.
  *
- * Deletes all files in a given @folder.
+ * @param camera a #Camera
+ * @param folder a folder
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_folder_delete_all (Camera *camera, const char *folder,
@@ -1115,15 +1132,14 @@ gp_camera_folder_delete_all (Camera *camera, const char *folder,
 }
 
 /**
- * gp_camera_folder_put_file:
- * \param camera: a #Camera
- * \param folder: a folder
- * \param file: a #CameraFile
- * \param context: a #GPContext
+ * Uploads a file into given \c folder.
  *
- * Uploads a file into given @folder.
+ * @param camera a #Camera
+ * @param folder a folder
+ * @param file a #CameraFile
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_folder_put_file (Camera *camera, const char *folder,
@@ -1143,16 +1159,15 @@ gp_camera_folder_put_file (Camera *camera, const char *folder,
 }
 
 /**
- * gp_camera_file_get_info:
- * \param camera: a #Camera
- * \param folder: a folder
- * \param file: the name of the file
- * \param info:
- * \param context: a #GPContext
+ * Retrieves information about a file.
  *
- * Retrieves information about a @file.
+ * @param camera a #Camera
+ * @param folder a folder
+ * @param file the name of the file
+ * @param info
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_file_get_info (Camera *camera, const char *folder, 
@@ -1162,7 +1177,7 @@ gp_camera_file_get_info (Camera *camera, const char *folder,
 	int result = GP_OK;
 	const char *mime_type;
 	const char *data;
-	long int size;
+	/* long int size; */
 	CameraFile *cfile;
 
 	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Getting file info for '%s' "
@@ -1196,6 +1211,7 @@ gp_camera_file_get_info (Camera *camera, const char *folder,
 	CRS (camera, gp_file_new (&cfile), context);
 	if (gp_camera_file_get (camera, folder, file, GP_FILE_TYPE_PREVIEW,
 						cfile, context) == GP_OK) {
+		unsigned long size;
 		info->preview.fields |= GP_FILE_INFO_SIZE | GP_FILE_INFO_TYPE;
 		gp_file_get_data_and_size (cfile, &data, &size);
 		info->preview.size = size;
@@ -1215,16 +1231,15 @@ gp_camera_file_get_info (Camera *camera, const char *folder,
 }
 
 /**
- * gp_camera_file_set_info:
- * \param camera: a #Camera
- * \param folder: a folder
- * \param file: the name of a file
- * \param info: the #CameraFileInfo
- * \param context: a #GPContext
- *
  * Sets some file properties like name or permissions.
  *
- * \return a gphoto2 error code
+ * @param camera a #Camera
+ * @param folder a folder
+ * @param file the name of a file
+ * @param info the #CameraFileInfo
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
  **/
 int
 gp_camera_file_set_info (Camera *camera, const char *folder, 
@@ -1242,17 +1257,16 @@ gp_camera_file_set_info (Camera *camera, const char *folder,
 }
 
 /**
- * gp_camera_file_get:
- * \param camera: a #Camera
- * \param folder: a folder
- * \param file: the name of a file
- * \param type: the #CameraFileType
- * \param camera_file: a #CameraFile
- * \param context: a #GPContext
+ * Retrieves a file from the #Camera.
  *
- * Retrieves a @file from the @camera.
+ * @param camera a #Camera
+ * @param folder a folder
+ * @param file the name of a file
+ * @param type the #CameraFileType
+ * @param camera_file a #CameraFile
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
- * \return a gphoto2 error code
  **/
 int 
 gp_camera_file_get (Camera *camera, const char *folder, const char *file,
@@ -1285,15 +1299,14 @@ gp_camera_file_get (Camera *camera, const char *folder, const char *file,
 }
 
 /**
- * gp_camera_file_delete:
- * \param camera: a #Camera
- * \param folder: a folder
- * \param file: the name of a file
- * \param context: a #GPContext
- *
- * Deletes the @file from a @folder.
+ * Deletes the file from \c folder.
  *
+ * \param camera a #Camera
+ * \param folder a folder
+ * \param file the name of a file
+ * \param context a #GPContext
  * \return a gphoto2 error code
+ *
  **/
 int
 gp_camera_file_delete (Camera *camera, const char *folder, const char *file,
@@ -1313,15 +1326,14 @@ gp_camera_file_delete (Camera *camera, const char *folder, const char *file,
 }
 
 /**
- * gp_camera_folder_make_dir:
- * \param camera: a #Camera
- * \param folder: the location where to create the new directory
- * \param name: the name of the directory to be created
- * \param context: a #GPContext
+ * Creates a new directory called \c name in the given \c folder.
  *
- * Creates a new directory called @name in given @folder.
+ * @param camera a #Camera
+ * @param folder the location where to create the new directory
+ * @param name the name of the directory to be created
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
- * \return a gphoto2 error code
  **/
 int
 gp_camera_folder_make_dir (Camera *camera, const char *folder,
@@ -1338,15 +1350,14 @@ gp_camera_folder_make_dir (Camera *camera, const char *folder,
 }
 
 /**
- * gp_camera_folder_remove_dir:
- * \param camera: a #Camera
- * \param folder: the folder from which to remove the directory
- * \param name: the name of the directory to be removed
- * \param context: a #GPContext
+ * Removes an (empty) directory called \c name from the given \c folder.
  *
- * Removes an (empty) directory called @name from the given @folder.
+ * @param camera a #Camera
+ * @param folder the folder from which to remove the directory
+ * @param name the name of the directory to be removed
+ * @param context a #GPContext
+ * @return a gphoto2 error code
  *
- * \return a gphoto2 error code
  */
 int
 gp_camera_folder_remove_dir (Camera *camera, const char *folder,
@@ -1363,17 +1374,44 @@ gp_camera_folder_remove_dir (Camera *camera, const char *folder,
 }
 
 /**
- * gp_camera_set_timeout_funcs:
- * \param camera a Camera
- * \param start_func
- * \param stop_func
- * \param data
+ * Gets information on the camera attached storage.
+ *
+ * @param camera a #Camera
+ * @param folder the folder from which to remove the directory
+ * @param name the name of the directory to be removed
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
+ */
+int
+gp_camera_get_storageinfo (
+	Camera *camera, CameraStorageInformation **sifs,
+	int *nrofsifs, GPContext *context)
+{
+	CHECK_NULL (camera && sifs && nrofsifs);
+	CHECK_INIT (camera, context);
+
+	CHECK_RESULT_OPEN_CLOSE (camera,
+		gp_filesystem_get_storageinfo (
+			camera->fs, sifs, nrofsifs, context
+		),
+		context
+	);
+	CAMERA_UNUSED (camera, context);
+	return (GP_OK);
+}
+
+/**
+ * @param camera a Camera
+ * @param start_func
+ * @param stop_func
+ * @param data
+ * @return a gphoto2 error code
  *
  * If your frontend has something like idle loops, it is recommended you
  * use #gp_camera_set_timeout_funcs in order to give the camera driver the
  * possibility to keep up the connection to the camera.
  *
- * \return a gphoto2 error code
  */
 void
 gp_camera_set_timeout_funcs (Camera *camera, CameraTimeoutStartFunc start_func,
@@ -1390,16 +1428,16 @@ gp_camera_set_timeout_funcs (Camera *camera, CameraTimeoutStartFunc start_func,
 
 
 /**
- * gp_camera_start_timeout:
- * \param camera a #Camera
- * \param timeout number of seconds that should pass between each call to
+ * @param camera a #Camera
+ * @param timeout number of seconds that should pass between each call to
  * 	     \c func
- * \param func the function that should be called each \c timeout seconds
- * \return The id of the background process or a gphoto2 error code
+ * @param func the function that should be called each \c timeout seconds
+ * @return The id of the background process or a gphoto2 error code
  *
  * This function should be called by the camera driver during camera_init()
  * if the camera needs to be sent messages periodically in order to prevent
  * it from shutting down.
+ *
  */
 int
 gp_camera_start_timeout (Camera *camera, unsigned int timeout,
@@ -1436,13 +1474,15 @@ gp_camera_start_timeout (Camera *camera, unsigned int timeout,
 
 
 /**
- * \brief Stop periodic calls to keepalive function.
- * \param camera a #Camera
- * \param id the id of the background process previously returned by 
- * 	gp_camera_start_timeout()
+ * Stop periodic calls to keepalive function.
+ *
+ * @param camera a #Camera
+ * @param id the id of the background process previously returned by 
+ *       #gp_camera_start_timeout
  *
  * Call this function in the camera driver if you want to stop a periodic
- * call to a function that has been started using gp_camera_start_timeout().
+ * call to a function that has been started using #gp_camera_start_timeout.
+ *
  */
 void
 gp_camera_stop_timeout (Camera *camera, unsigned int id)
